library(quantreg)
library(MASS)

## function for getting the level and slope estimator

innerfuncposx <- function(x)
{
res <- 0
if(abs(x) <= 1){
if(x >= 0) res <- 1/6 - 1/2 * x^2 + 1/3 * x^3
if(x < 0) res <- 1/6 - 1/2 * x^2 - 1/3 * x^3}
return(res)
}

## int_-1^1 v^2 K(v) dv = 1/6

levelslopeQR <- function(y,x,tau,h=n^(-1/5))
{
if(is.vector(x)) x <- as.matrix(x)
n <- length(y)
d <- length(x[1,])

thetaest <- rq(y~x+0,tau=tau)$coeff

## slopeest

G <- sapply(1:d,function(j)
{

Gpre <- sapply(1:n,function(z)
{
value <- (y[z] - drop(t(x[z,]) %*% thetaest) ) / (abs(x[z,j])*h)
result <- ifelse(x[z,j]!=0,innerfuncposx(value)*sign(x[z,j]),0)
result
}
)
#Gpre <- colMeans(Gpre * x) / (h*1/6)
colMeans(Gpre * x) / (h*1/6)
}
)

slopeest <- ginv(t(G)) %*% colMeans(x)

results <- c(thetaest,slopeest)
names(results)[1] <- "thetaest"
names(results)[d+1] <- "slopeest"

return(results)
}

m <- 1000
ypoints <- seq(-8,10,by=0.5)
epsilon <- 0.01
quantiles <- seq(epsilon,1-epsilon,by=0.01)
n <- 1000
h <- 1.5
densitycounter <- matrix(0,nrow=m,ncol=length(ypoints))

for(j in 1:m)
{
set.seed(j)
print(j)
u <- rlogis(n)
x <- rchisq(n,1)
y <- x+u*(1+x)
x <- cbind(rep(1,n),x)

estimatorsQR <- matrix(0,ncol=4,nrow=length(quantiles))
for(i in 1:length(quantiles))
{
#print(i)
estimatorsQR[i,] <- levelslopeQR(y,x,quantiles[i],h)
}

sums1QR <- estimatorsQR[,1:2] %*% c(1,1)
us <- epsilon+sapply(ypoints,function(y) mean(sums1QR <= y)) * (1-2*epsilon)

estimatorsQR <- matrix(0,ncol=4,nrow=length(us))
for(i in 1:length(us))
{
#print(i)
estimatorsQR[i,] <- levelslopeQR(y,x,us[i],h)
}
sums2QR <- estimatorsQR[,3:4] %*% c(1,1)
densityQR <- 1/sums2QR
densitycounter[j,] <- densityQR
}

write.table(densitycounter,"QR_density_estimation_n1000h15.txt")
ypoints <- seq(-8,10,by=0.5)
plotvalues <- rbind(apply(densitycounter,2,function(x) quantile(x,0.05)),apply(densitycounter,2,function(x) quantile(x,0.5)),apply(densitycounter,2,function(x) quantile(x,0.95)),sapply(ypoints, function(x) dlogis((x-1)/2)/2))
rownames(plotvalues) <- c("0.05 quantile","0.5 quantile","0.95 quantile","true value")
colnames(plotvalues) <- ypoints

pdf("QR_density_estimation_n1000h15.pdf",width=10,height=6)
par(las=1)
par(mai=c(.55, 0.6, 0.2, 0.1))
#par(cex.lab=1.5,cex.axis=1.5,mar=c(5,5,4,2) + 0.1,lwd=2)
plot(ypoints,plotvalues[1,],xlab="",ylab="",type="l",lty=2,pch=1,ylim=c(0,0.155))
par(new=T)
plot(ypoints,plotvalues[2,],xlab="",ylab="",type="p",pch=4,ylim=c(0,0.155))
par(new=T)
plot(ypoints,plotvalues[3,],xlab="",ylab="",type="l",lty=2,pch=3,ylim=c(0,0.155))
par(new=T)
plot(ypoints,plotvalues[4,],xlab="",ylab="",type="l",pch=4,ylim=c(0,0.155))
#legend("topright",c("0.05 quantile","0.5 quantile","0.95 quantile","true value"),pch=c(1,2,3,4),pt.cex=1.5)
dev.off()

densitycounter <- read.table("QR_density_estimation_n1000h15.txt")