library(quantreg)
library(MASS)

## function for getting the level and slope estimator

innerfuncposx <- function(x)
{
res <- 0
if(abs(x) <= 1){
if(x >= 0) res <- 1/6 - 1/2 * x^2 + 1/3 * x^3
if(x < 0) res <- 1/6 - 1/2 * x^2 - 1/3 * x^3}
return(res)
}

## int_-1^1 v^2 K(v) dv = 1/6

levelslopeQR <- function(y,x,tau,h=n^(-1/5))
{
if(is.vector(x)) x <- as.matrix(x)
n <- length(y)
d <- length(x[1,])

thetaest <- rq(y~x+0,tau=tau)$coeff

## slopeest

G <- sapply(1:d,function(j)
{

Gpre <- sapply(1:n,function(z)
{
value <- (y[z] - drop(t(x[z,]) %*% thetaest) ) / (abs(x[z,j])*h)
result <- ifelse(x[z,j]!=0,innerfuncposx(value)*sign(x[z,j]),0)
result
}
)
#Gpre <- colMeans(Gpre * x) / (h*1/6)
colMeans(Gpre * x) / (h*1/6)
}
)

slopeest <- solve(t(G)) %*% colMeans(x)

results <- c(thetaest,slopeest)
names(results)[1] <- "thetaest"
names(results)[d+1] <- "slopeest"

return(results)
}


m <- 10000
tautau <- c(0.1)
nn <- c(1000,4000)
hh <- c(0.5,0.7,0.9,1.1,1.3,1.5,1.7,2,3)
overallcounter <- matrix(0,ncol=15,nrow=(length(nn)*length(hh)*length(tautau)))
colnames(overallcounter) <- c("tau","n","Bandwidth","Bias L1","Bias L2","Bias S1","Bias S2","Var L1","Var L2","Var S1","Var S2","MSE L1","MSE L2","MSE S1","MSE S2")

www <- 1

overall <- 1
for(t in 1:length(tautau))
{
ourtau <- tautau[t]
for(a in 1:length(nn))
{
n <- nn[a]
for(b in 1:length(hh))
{
bandwidthexponent <- hh[b]
counter <- matrix(0,nrow=m,ncol=4)
for(i in 1:m)
{
print(i)
set.seed(i)
u <- rlogis(n)
x <- rchisq(n,1)
y <- x+u*(1+x*www)
x <- cbind(rep(1,n),x)

result <- levelslopeQR(y,x,ourtau,h=bandwidthexponent)
counter[i,] <- result
}
results <- c( sapply(1:4,function(x) mean(counter[,x])) ,sapply(1:4,function(x) var(counter[,x])))
resultsMSE1 <- results[5]+(results[1]-qlogis(ourtau))^2
resultsMSE2 <- results[6]+(results[2]-www*qlogis(ourtau)-1)^2
resultsMSE3 <- results[7]+(results[3]-1/dlogis(qlogis(ourtau)))^2
resultsMSE4 <- results[8]+(results[4]-1/dlogis(qlogis(ourtau)))^2
resultsMSE <- c(ourtau,n,bandwidthexponent,results[1]-qlogis(ourtau),results[2]-www*qlogis(ourtau)-1,results[3]-1/dlogis(qlogis(ourtau)),results[4]-1/dlogis(qlogis(ourtau)),results[5],results[6],results[7],results[8],resultsMSE1,resultsMSE2,resultsMSE3,resultsMSE4)
overallcounter[overall,] <- resultsMSE
print(round(overallcounter,3))
overall <- overall+1
}
}
}

overallcounter <- round(overallcounter,3)
write.table(overallcounter,"QR_paper_approach_with_covariate.txt",sep="&",row.names=F,eol="\\\ \n")

print(overallcounter)
##

## overallcounter <- read.table(file.choose(),header=T,sep="&")

library(xtable)
outtable<-xtable(overallcounter[,-c(1,2,4,5,8,9,12,13)],digits=3)
print(outtable,include.rownames=FALSE)